<?php
/*
Plugin Name: Side Matter
Plugin URI: http://wordpress.org/extend/plugins/side-matter/
Description: Generates footnotes in the sidebar, then magically positions them alongside their in-text references using the [ref] shortcode and a moveable widget.
Version: 0.5
Author: Christopher Setzer
License: GPLv2
*/

/*
Copyright (C) 2013  Christopher Setzer

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

$side_matter = new Side_Matter; // Instantiate class Side_Matter

class Side_Matter {

	public $side_matter_notes = array(); // Array of annotations to place in widget

	public function Side_Matter() { // Construct class
		add_shortcode( 'ref', array( &$this, 'ref_shortcode' ) ); // Register [ref] shortcode
		add_action( 'widgets_init', array( &$this, 'register_side_matter_widget' ) ); // Register widget
		add_action( 'wp_enqueue_scripts', array( &$this, 'enqueue_side_matter_script' ) ); // Enqueue script and stylesheet
		add_action( 'list_side_matter_notes', array( &$this, 'side_matter_list' ) ); // Attach function side_matter_list() to hook list_side_matter_notes() in widget
	}

	public function ref_shortcode( $atts, $content = null ) { // Handler function for [ref] shortcode
		$note_content = $content; // Capture shortcoded text
		$n = $this->process_notes( $note_content ); // Send note content to process_notes(), which will return current count of notes
		if ( is_single() || is_page() ) { // Only return in-text reference number on a single post or page
			return '<a id="ref-' . $n . '" class="side-matter side-matter-ref" href="#note-' . $n . '"><sup class="side-matter side-matter-sup">' . $n . '</sup></a>'; // Reference syntax: <a id="ref-1" class="side-matter-ref" href="#note-1"><sup>1</sup></a>
		} else {
			return; // Return nothing if not on a single post or page
		}
	}

	public function process_notes( $note = '' ) { // Add $note_content (here, $note) from ref_shortcode() to array $side_matter_notes, then return current count of notes to ref_shortcode()
		$this->side_matter_notes[] = $note; // Add $note to end of array
		return count( $this->side_matter_notes ); // Return count of notes
	}

	public function side_matter_list() { // Generate HTML list of notes to be displayed in sidebar widget
		if ( is_single() || is_page() ) { // Only display widget content if on a single page or post
			$notes = '<div id="side-matter-widget" class="side-matter"><ol id="side-matter-list" class="side-matter">'; // Begin building HTML list of notes
			if ( $this->side_matter_notes != null ) { // Only execute loop if $side_matter_notes contains notes
				foreach ( $this->side_matter_notes as $note_key => $note ) {
					$n = $note_key + 1;
					$notes .= '<a id="note-' . $n . '" class="side-matter"></a><li id="note-item-' . $n . '" class="side-matter side-matter-notes">' . $note . '</li>'; // Build a list item for every note
				}
			$notes .= '</ol></div>'; // End HTML list of notes
			echo $notes;
			} else {
				return;
			}
		}
	}

	public function register_side_matter_widget() { // Register widget with WordPress
		register_widget ( 'Side_Matter_Widget' );
	}

	public function enqueue_side_matter_script() { // Enqueue script and stylesheet if on a single post or page
		if ( is_single() || is_page() ) {
			wp_enqueue_script( 'side-matter', plugins_url( 'side-matter.js', __FILE__ ), array( 'jquery' ), null, true );
			wp_enqueue_style( 'side-matter', plugins_url( 'side-matter.css', __FILE__ ) );
		} else {
			return;
		}
	}

}

class Side_Matter_Widget extends WP_Widget {

	public function Side_Matter_Widget() { // Construct widget with description for Widgets page
		parent::__construct( false, 'Side Matter', array( 'description' => __( 'Display Side Matter annotations in an ordered list' ) ) );
	}

	public function widget( $args, $instance ) { // Content to display within widget
		if( is_single() || is_page() ) {
			do_action( 'list_side_matter_notes' ); // Action hook for function side_matter_list()
		}
	}

}